{ AIF.PAS, (C) 1993, Jay Shampur.

  This code was written and compiled with Borland Pascal 7.0.  The code
  has been tested both in real mode and as a DLL.  The DLL portion was
  tested only within Windows.

  The source code herein is written and copyrighted by Jay Shampur, 1993.
  However, the code may be used in your application freely with only one
  obligation: The target program and associated documents must contain
  the line  "Includes code written by Jay Shampur".

  No financial consideration is warranted or requested.

  If any improvements are made, please send the make the appropriate
  comments in the code and if appropriate, I would appreciate the
  opportunity to peruse the code.  Please e-mail me at 71521,21 on
  Compuserve.
  }

{
  This file may be compiled to a TPU, TPP or DLL file.  By default, it
  is set to compile to a DLL.  If you wish to compile to a TPU file
  remove the period from the line defining REALMODE and set the target
  as appropriate.

  All routines in this library will return an errorcode, ecFileIo, in
  case of any errors during file input or output.  If the operation is
  successfull, the ecSitNormal is returned.  Otherwise, appropriate
  error codes are returned.  See AIFINT.PAS for the CONSTANT definitions
  for these error codes.
  }


{.DEFINE RealMode}

{$IFNDEF RealMode}
library AIF;
Uses AIFInt, Strings;
{$V-,F+,I-}

{$ELSE}
{$V-,F+,I-,X+}
Unit AIFId;

Interface

Uses AIFInt;

Function GetNextAIFValue(CONST   S : pChar;
                         CONST fid : tAIFFileId):LongInt;

Function AIFFileIdExist(CONST S   : Pchar;
                        CONST Fid : tAIFFileId):LongInt;

Function ChangeAIFFileIdRec(CONST              S : pChar;
                            CONST OldFid, NewFid : tAIFFileId;
                            CONST       NewCount : LongInt):Integer;

Function DeleteAIFFileIdRec(CONST   S : pChar;
                            CONST Fid : tAIFFileId):Integer;

Function AddAIFFileIdRec(CONST      S : Pchar;
                         CONST    Fid : tAIFFileId;
                         CONST lCount : LongInt):Integer;

Function CreateAIFFile(CONST S : pChar):Integer;

Implementation
{$ENDIF}

{ This following two functions, OpenFile and GetAIFRecPos
  are used internally
  }

Function OpenFile(pFile : pAIFFile; CONST S:pChar):Boolean;
begin
   Assign(pfile^, S);
   Reset(pfile^);
   OpenFile := (IoResult = 0);
end;

Function GetAIFRecPos( pFile : pAIFFile;
                       fid   : tAIFFileId;
                   Var rbuf  : tAIFRec):LongInt;
{ Before this function is called, AIFFile must be open.
  If Fid is not found, filepointer is left at EOF.  If found,
  file pointer is left at the first byte of record after the
  end of the required record. Function is not interfaced }
var
   fPos : LongInt;
begin
   fPos := 0;  { Remember, the first component of a file is ZERO }
   GetAIFRecPos := 0;
   Read(pFile^, rBuf);   { Read and swallow Header  }
   While Not EOF(pfile^) do begin
      Read(pFile^, rbuf);
      Inc(fPos);
      if StrIcomp(rbuf.fileid, fid)=0 then begin { required file id found }
         GetAIFRecPos := fPos;
         exit;
      end;
   end;
end;

{ This function will return the next available ID as a longint for
  the associated fileid, if found.  Otherwise, the error code,
  ecFileIdNotFound is returned.
  }
Function GetNextAIFValue( CONST S   : pChar;
                          CONST fid : tAIFFileId):LongInt;
                        {$IFNDEF RealMode}export;{$ENDIF}
Var
  buf  : tAIFRec;
  fPos : LongInt;
  f    : tAIFFile;

begin
   If OpenFile(@F, S) then begin
      fPos := GetAIFRecPos(@F, fid,Buf);
      if fPos >0  then begin
         Inc(buf.LastAssigned);
         GetNextAIFValue := buf.LastAssigned;
         Seek(F, fPos);  {Write updated LastAssigned back to file and exit}
         Write(F, buf);
      end
      else
         GetNextAIFValue := ecFileIdNotFound;
      close(F);
   end
   else
      GetNextAIFValue := ecFileIo;
end;

{ CreateAIFFile must be called when the unit is used for the first
  time or if there is need to create more than one UniqueID file.
  Parameter is the filename to create with the complete path.  If a
  file by the same name exists, the file is not created and the function
  returns ecFileExists. Otherwise, if there is an error, ecFileCreate is
  returned.
  }

Function CreateAIFFile( CONST S : pChar):Integer;
                             {$IFNDEF RealMode}export;{$ENDIF}
var
  Buf : tAIFRec;
  f : tAIFFile;
begin
   CreateAIFFile := ecSitNormal;
   If OpenFile(@F,S) then begin
      Close(F);
      CreateAIFFile := ecFileExists;
   end
   else begin
      ReWrite(f);
      If IoResult <> 0 then
         CreateAIFFile := ecFileCreate
      else begin
         StrCopy(Buf.FileId,'UNIQUEID');
         Buf.LastAssigned := 0;
         Write(f,Buf);   { Write dummy header record }
         Close(f);
      end;
   end;
end;

{ For each file that contains an AIF and tracked by this library, there need to be
  a corresponding record in the AIF file.  The following function will add the record
  to the file.  Takes the filename to operate on, an id string for the file containing
  the AIF and the starting value.
  }
Function AddAIFFileIdRec( CONST      S : pChar;
                         CONST    Fid : tAIFFileId;
                         CONST lCount : LongInt):Integer;
                         {$IFNDEF RealMode}export;{$ENDIF}
var
  fPos : LongInt;
  buf  : tAIFRec;
  f    : tAIFFile;

begin
   AddAIFFileIdRec := ecSitNormal;
   If OpenFile(@F,S) then begin
      fPos := GetAIFRecPos(@F, Fid, buf);
      If fPos <= 0 then begin
         StrCopy(Buf.FileId,StrUpper(@Fid));
         Buf.LastAssigned := lCount;
         Write(F, Buf);
      end
      else
         AddAIFFileIdRec := ecFileIdExists;
      Close(F);
   end
   else
      AddAIFFileIdRec := ecFileIo;
end;

{ The following function checks if a fileid exists, but does not update
  the record.  If the fileid exists, the appropriate LASTASSIGNED is returned
  otherwise, ecFileIdNotFound is returned.
}
Function AIFFileIdExist( CONST S : Pchar;
                      CONST Fid : tAIFFileId ):LongInt;
                      {$IFNDEF RealMode}export;{$ENDIF}
Var
   F    : tAIFFile;
   buf  : tAIFRec;
begin
   AIFFileIdExist := ecFileIdNotFound;
   If (OpenFile(@F,S)) And (GetAIFRecPos(@F, Fid,Buf)>0) then
         AIFFileIdExist := Buf.LastAssigned;
   close(F);
   If IoResult=0 then ; { Throw away IoResult }
end;

{ If any file in the database is removed, the corresponding record in the
  AIF file can also be removed.  Call this function with the name of the
  AIF file and the file id to remove.
  }
Function DeleteAIFFileIdRec ( CONST   S : pChar;
                              CONST Fid : tAIFFileId ):Integer;
                             {$IFNDEF RealMode}export;{$ENDIF}
var
   fPos : LongInt;
   buf  : tAIFRec;
   f    : tAIFFile;

begin
   DeleteAIFFileIdRec := ecSitNormal;
   If Openfile(@F,S) then begin
      fPos := GetAIFRecPos(@F,fid,buf);
      if fPos <= 0 then
         DeleteAIFFileIdRec := ecFileIdNotFound
      else begin
         StrCopy(Buf.FileId,#0#0);
         Seek(F, fPos);
         Write(F, buf);
      end;
      Close(F);
   end
   else
      DeleteAIFFileIdRec := ecFileIo;
end;

{ For any reason, if the id of the file being tracked needs to be changed
  or if the AIF value for any file needs to be changed, call this function
  with the appropriate arguments
  }
Function ChangeAIFFileIdRec( CONST              S : pChar;
                            CONST OldFid, NewFid : tAIFFileId;
                            CONST NewCount       : LongInt):Integer;
                            {$IFNDEF RealMode}export;{$ENDIF}
Var
  fPos : LongInt;
  buf  : tAIFRec;
  f    : tAIFFile;
begin
   If OpenFile(@F,S) then begin
      fPos := GetAIFRecPos(@F, OldFid, buf);
      If fPos = 0 then
         ChangeAIFFileIdRec := ecFileIdNotFound
      else begin
         StrCopy(Buf.FileId,StrUpper(@NewFid));
         Buf.LastAssigned := NewCount;
         Seek(F, fPos);
         Write(F, Buf);
         ChangeAIFFileIdRec := ecSitNormal;
         Close(F);
      end;
   end
   else
      ChangeAIFFileIdRec := ecFileIo;
end;

{$IFNDEF RealMode}
exports
   CreateAIFFile index 1,
   GetNextAIFValue index 2,
   ChangeAIFFileIdRec index 3,
   AddAIFFileIdRec index 4,
   DeleteAIFFileIdRec index 5,
   AIFFileIdExist index 6;
{$ENDIF}

begin
end.
